/*
******************************************************************************

   Demo of the BLT extension.

   Authors:	2001 Christoph Egger	[Christoph_Egger@t-online.de]
  
   This code is placed in the public domain and may be used freely for any
   purpose.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <ggi/ggi.h>
#include <ggi/blt.h>


static int do_demo(ggi_visual_t vis)
{
	ggi_color col;
	ggi_mode mode;
	int width, height;
	ggiBlt_t blt;
	ggiBlt_source_t bob;

	col.r = col.g = col.b = 65535;
	ggiSetGCForeground(vis, ggiMapColor(vis, &col));

	/* Check what mode we actually got */
	ggiGetMode(vis, &mode);

	width = mode.visible.x;
	height = mode.visible.y;

	/* Set up the random number generator. */
	srandom(time(NULL));

	blt = ggiBltCreate(vis, GA_STORAGE_SWAP, 50, 0, 0);
	if (blt == NULL) ggiPanic("Couldn't get a blit.\n");

	bob = ggiBltCreateBob(vis, blt, GA_STORAGE_SWAP, 
			      100, 100, 1, GT_16BIT);
	if (bob == NULL) ggiPanic("Couldn't get a blit source.\n");

        ggiBltSetIdx(blt, 0);
	ggiBltPutBlit(blt, bob, 100, 100, 100, 100);
	fprintf(stderr,"here.\n");

	return 0;
}	/* do_demo */



int main(void)
{
	ggi_visual_t vis;
	ggiGA_resource_list reslist;
	reslist = NULL;

	/* Initialize LibGGI */
	if (ggiInit() != 0) {
		fprintf(stderr, "Unable to initialize LibGGI\n");
		exit(1);
	}
	
	/* Initialize BLT extension */
	if (ggiBltInit() != 0) {
		ggiPanic("Unable to initialize LibBLT extension\n");
	}

	/* Open the default visual */
	if ((vis = ggiOpen(NULL)) == NULL) {
		ggiPanic("Unable to open default visual\n");
	}

	if (ggiSetSimpleMode(vis, GGI_AUTO, GGI_AUTO, 1, GT_AUTO)) {
		ggiPanic("Could not set mode\n");
	}
	
	/* Turn on asynchronous mode (which should always be used) */
	ggiSetFlags(vis, GGIFLAG_ASYNC);
	

	/* Attach the BLT extension to the visual */
	if (ggiBltAttach(vis, &reslist) < 0) {
		ggiPanic("Unable to attach BLT extension to visual\n");
	}


	/* Try the API functions */
	do_demo(vis);

	/* Detach extension from visual */
	ggiBltDetach(vis);

	/* Close visual */
	ggiClose(vis);

	/* Deinitialize BLT extension */
	ggiBltExit();

	/* Deinitialize LibGGI */
	ggiExit();

	return 0;
}
