/*
******************************************************************************

   Libbuf implementation for "stubs" target -- API functions.
  
   Copyright (c) Wed Aug 29 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu
        Christoph Egger         Christoph_Egger@t-online.de
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.




******************************************************************************
*/

#include <stdio.h>
#include "stubsbuf.h"
#include <ggi/gg.h>
#include <string.h>

/* _All_ non-local (ie not declared 'static') functions and variables _must_
   be prefixed with the extension name, and for sublibs also with a unique
   sublib identifier. This is to keep the namespace clean on systems where
   all symbols are exported by default.
*/

/*-* API Implementation */

ggiBuf_t BUF_Stubs_ggiBuf_tie(ggi_visual_t vis, 
			      ggiGA_resource_handle motor_h,
			      ggiGA_resource_handle carb_h, 
			      ggiGA_resource_handle tank_h,
			      int narg, void *args)
{
	const char			*fname;
	ggiBuf_t			buf;
	struct ggiGA_resource_props	*props;
	bufpriv 			*priv;
	ggiGA_resource_handle		*ga;
	ggi_ll_ststate_t		ststate;

	priv = LIBGGI_BUFEXT(vis);

	/* Check is we actually own the resource we are being asked to tie */
	LIB_ASSERT(ggiGAGetVisual(tank_h) == vis);
	if (ggiGAGetVisual(tank_h) != vis) goto err0;

	/* TODO: Check some other stuff about the resources.  Note some
	 * checks have already been done in stubs.c and should be done by
	 * any other function calling us directly.
	 */

	props = ggiGAGetProperties(tank_h);

	/* TODO: Check that we have code to deal with this type/subtype */

	buf = _buf_alloc_def_llobj();
	if (buf == NULL) goto err0;

	ga = GA_LL(buf);
	ga[2] = motor_h;	/* Ordered thus because tank most accessed. */
	ga[1] = carb_h;
	ga[0] = tank_h;
	ga[3] = NULL;		/* ...already, but just to be clear. */

	ststate = GGI_LL_STSTATE(buf);

	/* TODO: handle narg/qty or (size_t)*args/tank.buf_snap as per docs */

#warning TODO: Check if props->sub.tank.db is valid, and do something if not.
#warning TODO: handle narg/qty or (size_t)*args/tank.buf_snap as per docs

	ststate->read.addr = ststate->display.addr = props->sub.tank.read;
	ststate->write.addr = props->sub.tank.write;
	ststate->bitdepth = GT_SIZE(props->sub.tank.gt);
	ststate->w = ststate->stride = props->size.area.x;
	ststate->h = props->size.area.y;

	switch (ggiGAGetType(tank_h)) {
	case GA_RT_BUFFER_ZBUFFER:
		ggiBufSetGCFlags(buf, BUF_CMP_LTE);
		ggiBufSetGCForeground(buf, (1 << ststate->bitdepth) - 1);
		ggiBufSetGCBackground(buf, 0);
		break;

	case GA_RT_BUFFER_ABUFFER:
		ggiBufSetGCFlags(buf, BUF_BLEND_OVER);
		ggiBufSetGCForeground(buf, (1 << ststate->bitdepth) - 1);
		ggiBufSetGCBackground(buf, 0);
		break;

	default:
		goto err1;
	}	/* switch */

	if (!priv->def_ops.usage) {
		fname = ggMatchConfig(priv->cfghand, "storage-stubs", NULL);
		if (fname == NULL) goto err1;
#warning error checking here.
		ggiExtensionLoadDL(vis, fname, NULL,
				   &(priv->def_ops), "STORAGEdl_");
	}	/* if */
	priv->def_ops.usage++;
	memcpy(GGI_LL_STOPS(buf), &(priv->def_ops), sizeof(priv->def_ops));
	return(buf);

 err1:
	_buf_free_def_llobj(buf);
 err0:
	BUFDPRINT("_tie failed.\n");
	return NULL;
}	/* BUF_Stubs_ggiBuf_tie */


int BUF_Stubs_ggiBuf_untie(ggi_visual *vis, ggiBuf_t *buf,
			   ggiGA_resource_handle carb_h, 
			   ggiGA_resource_handle tank_h)
{
	bufpriv 			*priv;

	priv = LIBGGI_BUFEXT(vis);

	/* TODO: Sync here?  Or just doc that user has to? */

	LIB_ASSERT(priv->def_ops.usage);
	if (!--(priv->def_ops.usage)) {
	  priv->def_ops.usage = 0; /* Keep gcc happy */
	  /* TODO: unload the default storage ops library. */
	}
	_buf_free_def_llobj(*buf);

	return GGI_OK;
}	/* BUF_Stubs_ggiBuf_untie */


int BUF_Stubs_getapi(ggi_visual_t vis,int num, char *apiname ,char *arguments)
{
	char zstr[4], astr[4], fstr[150], *ptr;
	bufpriv *priv;
	int rc;
	int flags = 0;

	priv = LIBGGI_BUFEXT(vis);
	zstr[0] = astr[0] = fstr[0] = '\0';

	/* We call libggi's original getapi function for this visual, and 
	 * try to do our best to replace any format strings with 
	 * libbuf-style renderers.
	 */
	rc = priv->getapi(vis, num, apiname, arguments);
	if (rc != 0) return rc;
	if (num == 0) return rc;

	apiname[255] = '\0'; /* color me me paranoid */	
	ptr = apiname + strlen(apiname);

	if (priv->achannel || (priv->abuf != NULL)) {
	  	if ((*(ptr - 2) == '-') || (*(ptr - 3) == '-')) {
			flags |= BUF_GETAPI_GIVE_PIXFMT;
		}	/* if */
		if (priv->abuf == NULL) flags |= BUF_GETAPI_ALPHA_USED;
		else sprintf(astr, "-a%d", 
			     (GGI_LL_STSTATE(priv->abuf))->bitdepth);
	}	/* if */
	if (priv->zbuf != NULL) {
		sprintf(zstr, "-z%d", (GGI_LL_STSTATE(priv->zbuf))->bitdepth);
	}	/* if */
	if (strstr(apiname, "-stubs") == NULL) {
		BUF_getapi_fstr(vis, fstr, flags);
	}	/* if */

	if (*(ptr - 2) == '-') ptr -= 2;
	if (*(ptr - 3) == '-') ptr -= 3;

	ptr += sprintf(ptr, "%s%s%s", fstr, astr, zstr);
	*ptr = '\0';
	return 0;

}	/* BUF_Stubs_getapi */


int BUF_Stubs_ggiBuf_clad(ggi_visual *vis, int flags, 
			  int numbufs, ggiBuf_t *bufs, ggiBuf_t *res)
{
	int err, idx, id;
	char sugname[256], args[256];
	bufpriv *priv;
	priv = LIBGGI_BUFEXT(vis);

	if (flags & BUF_CLAD_VISUAL) goto clad_vis;

	/* TODO: Clad other buffers */
	return GGI_EFATAL;

 clad_vis: 


	if (priv->is_clad) {
		BUFDPRINT("Visual is already clad, "
			  "must strip before cladding again.\n");
		return GGI_EFATAL;
	}
	priv->is_clad = 1;

	/* Save the current set of primitives. */
	if (priv->old_opdraw == NULL) {
		priv->old_opdraw = calloc(1, sizeof(struct ggi_visual_opdraw));
		if (priv->old_opdraw == NULL) return GGI_EFATAL;
	}	/* if */

	memcpy(priv->old_opdraw, vis->opdraw, sizeof(*vis->opdraw));

	for (idx = 0; idx < numbufs; idx++) {
		ggiGA_resource_handle *ga;
		enum ggiGA_resource_type gat;

		if (bufs[idx] == NULL) {
			/* This happens, when buffer allocation failed! */
			continue;
		}	/* if */

		ga = GA_LL(bufs[idx]);
		LIB_ASSERT(ga);
		gat = 0;
		while (*ga) {
		  gat = ggiGAGetType(*ga);
		  if ((gat & GA_RT_TYPE_MASK) == GA_RT_BUFFER) break;
		  gat = 0;
		  ga++;
		}

		if (gat == GA_RT_BUFFER_ZBUFFER) {
			priv->zbuf = bufs[idx];
		}	/* if */

		if (gat == GA_RT_BUFFER_ABUFFER) {
			if (0 /* TODO:test for alpha in pixel */) {
			} else {
				priv->abuf = bufs[idx];
			}	/* if */
		}	/* if */
	}	/* for */

	if ((priv->zbuf != NULL) || (priv->abuf != NULL)) {
		/* For now, until we have our own complete generic stubs,
		 * we need a copy of vanilla GGI generic-stubs.
		 */
		err = _ggiOpenDL(vis, "generic-stubs", NULL, NULL);
		if (err) {
			fprintf(stderr,"LibBuf: Can't open generic-stubs\n");
			return GGI_EFATAL;
		} else {
			BUFDPRINT("Success in loading generic stubs\n");
		}	/* if */
	}	/* if */

	/* Now we go get the rest of the api stack, with our special
	 * alpha/z/pixelformat in the suggest string courtesy of
	 * our special getapi implementation.  The first loop we
	 * do by hand for GGI opdraws, then the call to IndicateChange
	 * gets the extension operations. 
	 */
	for (id = 1; BUF_Stubs_getapi(vis,id,sugname,args) == 0; id++) {
		err = _ggiOpenDL(vis, sugname, args, NULL);
		if (err) {
			fprintf(stderr,"stubs-buf: Can't open the "
				"%s (%s) library.\n", sugname,args);
			if (id == 1) return GGI_EFATAL;	/* Must have stubs */
		} else {
			BUFDPRINT("Success in loading %s (%s)\n",sugname,args);
		}	/* if */
	}	/* for */
	ggiIndicateChange(vis, GGI_CHG_APILIST);
	
	return (0);
}	/* BUF_Stubs_ggiBuf_clad */


int BUF_Stubs_ggiBufStrip(ggi_visual *vis)
{
	bufpriv *priv;
	priv = LIBGGI_BUFEXT(vis);

	if (priv->old_opdraw != NULL) {

#warning TODO: Reload api stack fresh and use IndicateChange

		if (priv->is_clad) {
			/* Restore the visual's drawing primitives. */
			memcpy(vis->opdraw, priv->old_opdraw, 
			       sizeof(struct ggi_visual_opdraw));
		}	/* if */

		free(priv->old_opdraw);
		priv->old_opdraw = NULL;
	}       /* if */

	priv->abuf = NULL;
	priv->zbuf = NULL;
	priv->is_clad = 0;

	return GALLOC_OK;	
}	/* BUF_Stubs_ggiBufStrip */
